/*
 * Copyright 2012 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.hoau;

import at.jku.risc.stout.hoau.algo.AntiUnify;
import at.jku.risc.stout.hoau.algo.AntiUnifyProblem;
import at.jku.risc.stout.hoau.algo.DebugLevel;
import at.jku.risc.stout.hoau.data.EquationSystem;
import at.jku.risc.stout.hoau.data.InputParser;
import at.jku.risc.stout.hoau.data.NodeFactory;

/**
 * Some test cases
 * 
 * @author Alexander Baumgartner
 */
public class TestAntiUnify {

	public static void main(String[] args) {
		 //TermNode.printUserFriendly = false;
		 // NodeFactory.PREFIX_IndividualVar = "$";
		test("\\times,0,1,2,3.f(if(eq(n,0), then(1)), elseif(eq(n,1), then(times(1,1))), elseif(eq(n,2), then(times(2,times(1,1)))), elseif(eq(n,3), then(times(3,times(2,times(1,1)))))) =^="
				+ "\\substract,0,1,2,3.f(if(eq(n,0), then(0)), elseif(eq(n,1), then(subtract(0,1))), elseif(eq(n,2), then(subtract(subtract(0,1),2))), elseif(eq(n,3), then(subtract(subtract(subtract(0,1),2),3))))");
		test("\\times,0,1,2,3.if(eq(n,0), 1,     if(eq(n,1), times(1,1),            if(eq(n,2), times(times(1,1),2),                   if(eq(n,3), times(times(times(1,1),2),3))            )     ) ) =^= \\subtract,0,1,2,3.if(eq(n,0), 0,     if(eq(n,1), subtract(0,1),            if(eq(n,2), subtract(subtract(0,1),2),                   if(eq(n,3), subtract(subtract(subtract(0,1),2),3))           )     ) ) ");
		 test("\\x.f(b,x(y)) =^= \\x.f(a,x(b))");
		 test("f(x) =^= f(x)");
		 test("\\x.f(x,x) =^= \\x.f(a,x)");
		 test("\\x.f(b,x) =^= \\x.f(a,x)");
		 test("\\y:i2.\\z:i3. f:i2-i3-i1(y,z) = \\y:i2.\\z:i3. x:i3-i2-i1(z,y)");
		 test("\\x,y.f(f(g(x),y),f(g(y),x)) =^= \\x,y.f(h(y,g(x)),h(x,g(y)))");
		 test("\\x,y.f(x,y) =^= \\x,y.f(y,x)");
		 test("\\x:t1,y:t2.f(x,y) =^= \\x:t1,y:t2.f(y,x)"); // partially typed
		 test("\\x,y,z.g(f(x,z),f(y,z),f(y,x)) =^= \\x,y,z.g(h(y,x),h(x,y),h(z,y))");
		 test("\\x,y.f(\\z.g(z,y),g(x,y)) =^= \\x,y.f(\\z.h(y,z),h(y,x))");
		 test("\\x.f(\\y.g(x,y),\\y.g(y,x)) =^= \\x.f(\\y.h(x,y,x),\\y.h(y,x,y))");
		 test("\\x,y.f(x(x)) =^= \\x,y.f(y(y))");
		 test("\\x.f(\\y.g(y),x) =^= \\x.f(g(x),x)"); // eta expansion is done for unranked g?
		 test("\\x.f(\\y.g(y),x) =^= \\x.f(\\y.g(x,y),x)");
		 test("\\x,y.f(g(y),x) =^= \\x.f(g(x),x)"); // unranked f gets eta-expanded
		 test("\\x,y.f(g(y),x) =^= \\x.f(g(x))"); // unranked f gets eta-expanded
		 test("\\x,y.f(g(x),y) =^= \\x.f(g(x))");
		 test("\\x,y.f(x(g(x),y),x(g(y),x)) =^= \\x,y.f(x(y,g(x)),y(x,g(y)))");
		 test("(\\x.f(a, x(f, (b, c)))) \\x.\\y.g(x(y))");
		 test("\\a,b,c,d.f(\\a.a,\\b.b) =^= \\a,b,c,d.f(\\c.X(a,b,d),\\d.X(a,b,d))");
		 test("\\x,y,z. f(x(y,z),x(z,y)) =^= \\x,y,z. f(X(y,\\u.u), X(z,\\v.v))");
		 test("\\y.\\x.f(\\x.x,y) =^= \\x,y.f(\\y.y,y)");
		 test("\\y.\\x.z1(\\x.x,y) =^= \\x,y.z1(\\y.y,y)");
	}

	public static void test(String in) {
		try {
			EquationSystem<AntiUnifyProblem> eqSys = new EquationSystem<AntiUnifyProblem>() {
				@Override
				public AntiUnifyProblem newEquation() {
					return new AntiUnifyProblem();
				}
			};
			NodeFactory.resetCounter();
			InputParser parser = new InputParser();
			parser.tryFreeVarPrefix(in, "X", "Y", "Z", "V", "U");
			parser.tryBoundVarPrefix(in, "x", "y", "z", "v", "u");
			parser.parseEquationSystem(in, eqSys, 100);
			System.out.println(" Input: " + eqSys);
			AntiUnify au = new AntiUnify(eqSys, 100, DebugLevel.SIMPLE);
			au.antiUnify(true, System.out);
			System.out.println();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
