package at.jku.risc.stout.hoau;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.util.Map;
import java.util.Set;

import at.jku.risc.stout.hoau.algo.AntiUnify;
import at.jku.risc.stout.hoau.algo.AntiUnifyProblem;
import at.jku.risc.stout.hoau.algo.AntiUnifySystem;
import at.jku.risc.stout.hoau.algo.DebugLevel;
import at.jku.risc.stout.hoau.algo.JustificationException;
import at.jku.risc.stout.hoau.algo.PermEquiv;
import at.jku.risc.stout.hoau.algo.PermEquivProblem;
import at.jku.risc.stout.hoau.data.EquationSystem;
import at.jku.risc.stout.hoau.data.InputParser;
import at.jku.risc.stout.hoau.data.MalformedTermException;
import at.jku.risc.stout.hoau.data.NotNormalizableException;
import at.jku.risc.stout.hoau.data.atom.Variable;
import at.jku.risc.stout.hoau.util.ControlledException;
import at.jku.risc.stout.hoau.util.DataStructureFactory;

public class TestPaperExample {

	public static void main(String[] args) throws IOException,
			JustificationException, ControlledException {
		auTest();
		equivTest();
	}

	public static void auTest() throws NotNormalizableException,
			MalformedTermException, IOException, ControlledException {
		Reader in1 = new StringReader("\\x.f(b,a,a)"); // Use FileReader instead
		Reader in2 = new StringReader("\\x.g(a,x,b)"); // Use FileReader instead
		int maxReduce = 100;

		EquationSystem<AntiUnifyProblem> eqSys = new EquationSystem<AntiUnifyProblem>() {
			public AntiUnifyProblem newEquation() {
				return new AntiUnifyProblem();
			}
		};
		new InputParser().parseEquation(in1, in2, eqSys, maxReduce);

		new AntiUnify(eqSys, maxReduce, DebugLevel.SILENT) {
			public void callback(AntiUnifySystem res, Variable var) {
				System.out.println(res.getSigma().get(var));
			};
		}.antiUnify(false, null);
	}

	public static void equivTest() throws IOException, ControlledException {
		Reader in1 = new StringReader("f(x,y)"); // Use FileReader instead
		Reader in2 = new StringReader("f(y,x)"); // Use FileReader instead
		int maxReduce = 100;

		Set<Variable> dom = DataStructureFactory.$.newSet();
		dom.add(new Variable("x", null));
		dom.add(new Variable("y", null));
		Set<Variable> ran = DataStructureFactory.$.newSet();
		ran.add(new Variable("x", null));
		ran.add(new Variable("y", null));

		EquationSystem<PermEquivProblem> eqSys = new EquationSystem<PermEquivProblem>() {
			public PermEquivProblem newEquation() {
				return new PermEquivProblem();
			}
		};
		new InputParser().parseEquation(in1, in2, eqSys, maxReduce);

		Map<Variable, Variable> permutation = new PermEquiv(eqSys, dom, ran)
				.compute(DebugLevel.SILENT, null);
		System.out.println(permutation);
	}
}
