/*
 * Copyright 2012 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.hoau.algo;

import at.jku.risc.stout.hoau.data.Equation;
import at.jku.risc.stout.hoau.data.TermNode;

/**
 * This class represents a matching problem of two {@linkplain TermNode}s. This
 * sort of matching problems are used by the rule based system
 * {@linkplain PermEquivSystem}.
 * 
 * @author Alexander Baumgartner
 */
public class PermEquivProblem implements Equation {
	private TermNode left, right;

	@Override
	public String toString() {
		return left + " >> " + right;
	}

	@Override
	public TermNode getLeft() {
		return left;
	}

	@Override
	public TermNode getRight() {
		return right;
	}

	@Override
	public void setLeft(TermNode node) {
		left = node.reduceHedges();
	}

	@Override
	public void setRight(TermNode node) {
		right = node.reduceHedges();
	}

	@Override
	public PermEquivProblem clone() {
		try {
			PermEquivProblem clone = (PermEquivProblem) super.clone();
			clone.left = left.clone();
			clone.right = right.clone();
			return clone;
		} catch (CloneNotSupportedException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Abstracts the lambda bound variables of both sides of the equation. The
	 * top atom of both sides has to be a lambda atom!
	 */
	public void abstrLambda() {
		left.setAtom(null);
		right.setAtom(null);
		left = left.reduceHedges();
		right = right.reduceHedges();
	}
}