/*
 * Copyright 2012 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.hoau.data;

import java.io.IOException;
import java.io.Writer;
import java.util.List;

import at.jku.risc.stout.hoau.data.atom.BoundVariable;
import at.jku.risc.stout.hoau.data.atom.TermAtom;
import at.jku.risc.stout.hoau.data.atom.Variable;
import at.jku.risc.stout.hoau.util.DataStructureFactory;
import at.jku.risc.stout.hoau.util.Printable;

/**
 * A {@linkplain Hedge} is a sequence of {@linkplain TermNode}s.
 * 
 * @author Alexander Baumgartner
 */
public class Hedge extends Printable implements Cloneable {

	public static String PRINT_PARAM_START = "(";
	public static String PRINT_PARAM_SEPARATOR = ", ";
	public static String PRINT_PARAM_END = ")";
	protected List<TermNode> sequence = DataStructureFactory.$.newList();

	public boolean isEmpty() {
		return sequence.size() == 0;
	}

	@Override
	public boolean equals(Object other) {
		if (!(other instanceof Hedge))
			return false;
		Hedge oH = (Hedge) other;
		if (size() != oH.size())
			return false;
		for (int i = size() - 1; i >= 0; i--)
			if (!sequence.get(i).equals(oH.sequence.get(i)))
				return false;
		return true;
	}

	public void add(TermNode t) {
		sequence.add(t);
	}

	/*
	 * public void add(TermAtom at) { sequence.add(new TermNode(at, null)); }
	 */
	public TermNode get(int i) {
		return sequence.get(i);
	}

	public int size() {
		return sequence.size();
	}

	public void clear() {
		sequence.clear();
	}

	public Hedge newInstance() {
		return new Hedge();
	}

	public Hedge reduce(int count) throws NotNormalizableException {
		for (int i = 0, n = size(); i < n; i++)
			sequence.set(i, get(i).reduce(count));
		return this;
	}

	public void substitute(Variable from, TermNode to)
			throws NotNormalizableException {
		for (int i = 0, n = size(); i < n; i++)
			sequence.get(i).substitute(from, to);
	}

	public void substitute(BoundVariable from, BoundVariable to) {
		for (int i = 0, n = size(); i < n; i++)
			sequence.get(i).substitute(from, to);
	}

	@Override
	public Hedge clone() {
		Hedge clone = newInstance();
		clone.sequence = DataStructureFactory.$.newList(sequence.size());
		for (int i = 0, n = sequence.size(); i < n; i++)
			clone.sequence.add(sequence.get(i).clone());
		return clone;
	}

	@Override
	public void print(Writer out) throws IOException {
		print(out, false);
	}

	public void print(Writer out, TermAtom atom, boolean needLamdaBracket)
			throws IOException {
		atom.print(out);
		print(out, true);
	}

	private void print(Writer out, boolean friendlyOut) throws IOException {
		if (sequence.size() > 0) {
			out.append(PRINT_PARAM_START);
			for (int i = 0, n = sequence.size(); i < n; i++) {
				if (i > 0)
					out.append(PRINT_PARAM_SEPARATOR);
				if (friendlyOut)
					sequence.get(i).print(out, false);
				else
					sequence.get(i).print(out);
			}
			out.append(PRINT_PARAM_END);
		}
	}
}
