/*
 * Copyright 2012 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.hoau.data.atom;

import java.io.IOException;
import java.io.Writer;

/**
 * A {@linkplain TermAtom} which represents a variable. Variables can be typed.
 * 
 * @author Alexander Baumgartner
 */
public class Variable extends TermAtom {
	private String type;

	public Variable(String name, String type) {
		super(name);
		if (type != null)
			this.type = type.intern();
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof Variable) {
			Variable var = (Variable) obj;
			return super.equals(var) && type == var.type;
		}
		return false;
	}

	@Override
	public String getType() {
		return type;
	}

	@Override
	public boolean isBasicType() {
		return true;
	}

	@Override
	public void print(Writer out) throws IOException {
		print(out, getName());
	}

	protected void print(Writer out, String name) throws IOException {
		if (type != null)
			out.append(name).append(':').append(type);
		else
			out.append(name);
	}
}
